% quarter car model with trailing arm suspension - obstacle
clear all,  close all

% globals
global g s a b h r0
global mC mK mW ThetaK ThetaW
global Ts0 cs ds  cx cz dx
global v_V Amp Om Psi
global z_road fz_wheel

% vehicle data
qcm_data

% vehicle velocity [km/h --> m/s] 
v_V  = 80/3.6; 
% spectral density and waviness
Phi0 = 10e-6; w = 2; 
% range of frequencies and number of samples
Omin = 0.0628;  Omax = 62.83;  n = 200;

% calculate amplitudes and random phases
dOm = (Omax-Omin)/(n-1);
Om  = Omin:dOm:Omax;
Om0 = 1; 
Phi = Phi0.*(Om./Om0).^(-w);  Amp = sqrt(2*Phi*dOm);
Psi = 2*pi*rand(size(Om)); 

% initial states:  x0 = [ zC; betaK; phiW; zCd; betaKd; phiWd ]
x0 = [ 0; 0; 0; 0; 0; 0];

% simulation interval
t0=0; tE=6;
% adjust initial state to road
s_V = v_V*t0; u = sum( Amp.*sin(Om*s_V+Psi) ); x0(1)=u;
% perform time simulation
[tout,xout] = ode45(@qcm_f,[t0,tE],x0);

% get road height, wheel load and chassis acceleration
zr=zeros(size(tout)); fz=zeros(size(tout)); zdd=zeros(size(tout));
for i=1:length(tout)
  xdot = qcm_f(tout(i),xout(i,:)');
  zr(i)= z_road;  fz(i)=fz_wheel;  zdd(i)=xdot(4);
end

% mean value and standard deviation of wheel load
fz_m = mean(fz);  fz_std = std(fz);
disp(['wheel load: mean value =',num2str(fz_m)])
disp(['wheel load: standard deviation =',num2str(fz_std)])

% generate histogram of wheel load
nh=20; [nfz,hfz]=hist(fz,nh); 

% generate gaussian density function and adjust to histogramm
fzi=linspace(min(fz),max(fz),201);
pfz = exp(-(fzi-fz_m).^2./(2*fz_std.^2))./(fz_std*sqrt(2*pi));
pfz = pfz * (sum(nfz)*(max(fz)-min(fz))/nh);

% plot results
subplot(3,2,1), plot(tout,zr), grid on
                xlabel('time'), legend('road: z_R ')
subplot(3,2,2), plot(tout,xout(:,2)*180/pi), grid on
                xlabel('time'), legend('\beta [deg]')
subplot(3,2,3), plot(tout,xout(:,1)), grid on
                xlabel('time'), legend('chassis: z(t)')
subplot(3,2,4), plot(tout,xout(:,3)*180/pi), grid on
                xlabel('time'), legend('\phi [deg]')
subplot(3,2,5), plot(tout,fz/1000), grid on
                xlabel('time'), legend('wheel load F_z [kN]') 
subplot(3,2,6), barh(hfz/1000,nfz), colormap('white'), hold on
                plot(pfz,fzi/1000,'k--','Linewidth',2)
                legend('Histogram F_z','Gauss')              