% approximate static wheel load characteristic
clear all, close all  

% measured wheel load versus tire deflection
Fz = [ 0.0; 10.9;  21.5;  32.2;  46.1 ]; % wheel load [kN]
dz = [ 0.0; 15.5;  27.2;  36.4;  47.2 ]; % tire deflection [mm]
% least square approximation by parabola: Fz = a(1)*dz + a(2)*dz^2
A = [dz, dz.^2]; a = A\Fz;
% get deflections at payload FzN and double the payload 2*FzN
FzN = 35;  % tire payload (truck tire)
dz_FzN  = ( -a(1) + sqrt(a(1)^2+4*a(2)*FzN)   ) / (2*a(2)) ;
dz_2FzN = ( -a(1) + sqrt(a(1)^2+4*a(2)*2*FzN) ) / (2*a(2)) ;
% calculate stiffness at FzN and 2*FzN
c_z1 = ( a(1) + 2*a(2)*dz_FzN  );
c_z2 = ( a(1) + 2*a(2)*dz_2FzN );
% plot approximated characteristic and compare to measurements
d_z = linspace(0,2*dz_FzN,201);
F_z = a(1)*d_z + a(2)*d_z.^2;
plot(d_z,F_z,'r','Linewidth',1.5), hold on, plot(dz,Fz,'ok' ), grid on
xlabel('\Delta z [mm]'), ylabel('F_z [kN]')
title(['X31580 R22.5  c_z_1 =',num2str(c_z1),' kN/mm'])