% Wheel Tire Dynamics via simple quarter car model, Rill: 04.09.2010
clear all, close all

global  gravity al mass Theta r  dfx0 fxm sxm fxs sxs  vN  cx dx
global  t_Drive Drive_Torque  t_Brake Brake_Torque  dN
global  sx fxd tq_B tq_D

% chassis and wheel data
gravity = 9.81;     % [m/s^2] constant of gravity
al = 20/180*pi;     % [degree-->rad] grade angle
mass  = 400;        % [kg] chassis mass
Theta = 1.2;        % [kgm^2] inertia of wheel
r = 0.3;            % [m] wheel radius

% long. tire characteristic with linear wheel load influence
fzN  = 3100; fz = mass*gravity*cos(al);
dfx0 = 100000*fz/fzN;  % [N/-] initial incl. long. force char.
fxm  = 3200*fz/fzN;    % [N] maximum long. force
sxm  = 0.1;            % [-] sx where fx=fxm
fxs  = 3000*fz/fzN;    % [N] long. sliding force
sxs  = 0.8;            % [-] sx where fx=fxs
cx = 160000;           % [N/m] longitudinal tire stiffness
dx =  500;             % [N/(m/s)] longitudinal tire damping
vN=0.01;               % [m/s] fictitious velocity

% adjust "damping" constant in enhanced braking torque model to wheel/tire
dN = r * sqrt(cx*Theta);
% set driving torque (time [s], torque [Nm])
t_Drive      = [ 0.0   0.9   1.1   3.9  4.1  20.0 ];
Drive_Torque = [ 0.0   0.0   1.0   1.0  0.0   0.0  ]*fxm*r; 
% set braking torque (time [s], torque [Nm])
t_Brake      = [ 0.0  5.95  6.05  17.95   18.05   20.0 ];
Brake_Torque = [ 0.0  0.00  1.50   1.50    0.00    0.0 ]*fxm*r; 
% perform simulation 
tE = min(max(t_Drive),max(t_Brake)); % duration
x0=[ 0; 0; 0; 0]; % simple initial states
tic; [t,xout] = ode23(@Chassis_Wheel_Tire_f,[0,tE],x0); toc
% get additional output quantities
sxi=t; fxi=t; tqb=t; tqd=t; 
for i=1:length(t)
  xp = Chassis_Wheel_Tire_f(t(i),xout(i,:)');
  sxi(i)=sx; fxi(i)=fxd; tqb(i)=tq_B; tqd(i)=tq_D;
end    
% plot results    
subplot(2,2,1); hold on, grid on
 plot(t,xout(:,1),'k','Linewidth',1), plot(t,r*xout(:,2),'--r','Linewidth',1)
 title('Velocities: v and r*Omega')
subplot(2,2,2); hold on, grid on
 plot(t,xout(:,4),'k','Linewidth',1)
 title('vehicle position')
subplot(2,2,3); hold on, grid on
 plot(t,tqd,'k','Linewidth',1), plot(t,tqb,'--r','Linewidth',1)
 title('Driving and braking torque [Nm]')
subplot(2,2,4); hold on, grid on
 plot(t,fxi,'k','Linewidth',1)
 title('Longitudinal force [N]')
 
figure, semilogy(0.5*(t(1:length(t)-1)+t(2:length(t))),diff(t),'ok','MarkerSize',3)