% Eigenvalues wheel dynamic
clear all, close all

Theta = 1.2;      % [kgm^2] inertia of wheel
r = 0.3;          % [m] wheel radius (dynamic = static)
dfx0 = 100000;    % [N/-] initial incl. force char.
cx = 160000;      % [N/m] longitudinal tire stiffness
dx = 500;         % [N/(m/s)] longitudinal tire damping 
vN = 0.01;        % [m/s] fictitious velocity

% set velocity range and allocate space to speed up loop
vi=linspace(0,210/3.6,101); la1=zeros(size(vi));la2=la1; freq=la1;dratio=la1;

for i=1:length(vi)
% corresponding mass oscillator
  v = vi(i) + vN;
  m = Theta/r^2*(1+v*dx/dfx0); d = dx + v*cx*Theta/(r^2*dfx0); c = cx;
% eigenvalues
  la1(i) = -d/(2*m) + sqrt( (d/(2*m))^2 - c/m );
  la2(i) = -d/(2*m) - sqrt( (d/(2*m))^2 - c/m );
% frequency [Hz] and damping ratio [-]
  freq(i)   = imag(la1(i))/(2*pi);
  dratio(i) = d / ( 2*sqrt(c*m) );
end

% plots  
subplot(1,3,1), hold on, grid on, title('Eigenvalues'),axis equal 
plot(real(la1),imag(la1)), plot(real(la2),imag(la2))
subplot(1,3,2), hold on, grid on, title('Eigenfrequencies [Hz]')
plot(vi,freq)
subplot(1,3,3), hold on, grid on, title('Damping ratio [-]')
plot(vi,dratio)

% undamped eigenvalue at v=0
m = Theta/r^2*(1+vN*dx/dfx0); d = dx + vN*cx*Theta/(r^2*dfx0); c = cx;
la0 = sqrt(c/m)