function [ f, fos ] = tmy_fcombined( s, df0,fm,sm,fs,ss )
% combined tire force f, global derivative fos and maximum value fmax

% set initial slope to appropriate value if given one is too small
  df0loc = 0;   if sm > 0, df0loc = max(2*fm/sm, df0);  end

  if s > 0  &&  df0loc > 0  % normal operating conditions 
    if s > ss  % full sliding  
      f = fs;  fos = f/s;
    else
      if  s < sm  % adhesion
        p = df0loc*sm/fm - 2 ;  sn = s/sm;  dn = 1 + ( sn + p ) * sn ;   
        f = df0loc*sm*sn/dn;  fos = df0loc/dn;
      else  % adhesion --> sliding 
        a = (fm/sm)^2/(df0loc*sm); % parameter from 2. deriv. of f @ s=sm
        sstar = sm + (fm-fs)/(a*(ss-sm)); % connecting point      
        if sstar <= ss  % 2 parabolas (standard approach)
          if s <= sstar  % 1. parabola sm < s < sstar
            f = fm - a*(s-sm)^2;
          else           % 2. parabola sstar < s < ss
            b = a*(sstar-sm)/(ss-sstar);   f = fs + b*(ss-s)^2;
          end
        else   %  1 cubic function (just in case)
          sn = (s-sm)/(ss-sm) ;   f = fm - (fm-fs)*sn^2*(3.d0-2.d0*sn) ;
        end
        fos = f/s ;   % global derivative 
      end
    end
  else  
    f = 0; fos = df0loc;   %  defaults (s=0)
  end
  
end