function [ avw ...    % rotation matrix wheel rim / ref-sys
         , rvwv ...   % actual position of wheel center
         , del ...    % rotation angle arround king pin
         , pv ...     % partial velocities 
         ] = ...
 dblwb_kin( phi ...   % rotation angle of lower control arm
          , u ...     % rack displacement
          , dt ...    % model data
         )
% kinematics of a double whisbone axle with rack and pinion steering

% lower wishbone 
  rab = dt.rvbk-dt.rvak;  eab = rab / norm(rab);  eabs = skewsym(eab);
  eabeab = eab*eab';  eyeab = eye(3,3)-eabeab;
  aphi = eabeab + eyeab*cos(phi) + eabs*sin(phi);

% upper wishbone
  rcfk = dt.rvfk-dt.rvck;  rdfk = dt.rvfk-dt.rvdk; 
  racv=aphi*(dt.rvck-dt.rvak); rcdv=dt.rvdk-(dt.rvak+racv); 
  rde=dt.rvek-dt.rvdk; ede=rde/norm(rde); edeede=ede*ede'; edes=skewsym(ede);
  a = rcdv'*(eye(3,3)-edeede)*rdfk;  b = rcdv'*cross(ede,rdfk);
  c = -rcdv'*edeede*rdfk-0.5*(rdfk'*rdfk+rcdv'*rcdv-rcfk'*rcfk);
  psi = trigon(a,b,c);
  apsi = edeede + (eye(3,3)-edeede)*cos(psi) + edes*sin(psi);
  
% orientation of wheel body due to control arm motion
  rvcv = dt.rvak + racv;
  rdfv = apsi*(dt.rvfk-dt.rvdk); rvfv = dt.rvdk + rdfv; rcfv = rvfv - rvcv;  
  be = trigon(rcfk(1),rcfk(3),rcfv(1));
  abe = [ cos(be)  0  sin(be) ; ...
             0     1     0    ; ...
         -sin(be)  0  cos(be) ];   
  al = trigon(rcfv(2),rcfv(3),rcfk(2));
  aal = [ 1     0       0     ; ...
          0  cos(al) -sin(al) ; ...
          0  sin(al)  cos(al) ] ;

% rotation arround king pin
  rvrv = dt.rvrk + [ 0; u; 0 ];  rrcv = rvcv - rvrv;  rrcht=rrcv'*aal*abe;
  rrqk = dt.rvqk-dt.rvrk;  rcqk = dt.rvqk-dt.rvck;  rcfk = dt.rvfk-dt.rvck;  
  ecf = rcfk / norm(rcfk); ecfecf = ecf*ecf';  ecfs = skewsym(ecf);
  a = rrcht*(eye(3,3)-ecfecf)*rcqk;  b = rrcht*cross(ecf,rcqk);
  c = -( rrcht*ecfecf*rcqk + 0.5*(rrcv'*rrcv + rcqk'*rcqk - rrqk'*rrqk) );
  del=trigon(a,b,c);  adel=ecfecf+(eye(3,3)-ecfecf)*cos(del)+ecfs*sin(del);

% wheel body orientation and position 
  avw = aal*abe*adel;  rcwv = avw*(dt.rvwk-dt.rvck); rvwv = rvcv + rcwv;
  
% partial derivatives: dpsi/dphi, dal/dphi, dbe/dphi
  exvv=[1;0;0]; eyalv=aal*[0;1;0]; 
  a = [ cross(ede,rdfv)  -cross(exvv,rcfv)  -cross(eyalv,rcfv) ];
  c = a \ cross(eab,racv); dpsidphi=c(1); daldphi=c(2); dbedphi=c(3);

% partial derivatives: ddel/dphi, ddel/du
  eyvv=[0;1;0]; ecfv=rcfv/norm(rcfv);
  rcqv=avw*rcqk; rvqv=rvcv+rcqv; rrqv=rvqv-rvrv; 
  a = exvv*daldphi + eyalv*dbedphi; b = cross(eab,racv) + cross(a,rcqv);
  c = rrqv'*cross(ecfv,rcqv); ddeldphi = -rrqv'*b/c; ddeldu = rrqv'*eyvv/c;

% partial angular and partial velocities of wheel carrier  
  dodphi=exvv*daldphi+eyalv*dbedphi+ecfv*ddeldphi;  dodu=ecfv*ddeldu;
  dvdphi=cross(eab,racv)+cross(dodphi,rcwv); dvdu = cross(dodu,rcwv);
  pv = [ dodphi dodu dvdphi dvdu ];  
  
end