% sweep excitation of a damper/topmount combination
clear all, close all

% globals 
global  damper_inv  c   p  q  amp  

% damper characteristics F_D=F_D(v) as lookup table [ v  F_D ]
ffr = 100;  % define friction force and extend lookup table appropriately
damper_table = ...
[ -1.5080  -3500-ffr; ... 
  -1.1310  -2800-ffr; ...
  -0.5655  -1500-ffr; ...
  -0.4524  -1250-ffr; ...
  -0.3016  -1000-ffr; ...
  -0.1508   -650-ffr; ...
  -0.0377   -200-ffr; ...
   0.0000      0-ffr; ...
   0.0000      0+ffr; ...
   0.0377    100+ffr; ...
   0.1508    150+ffr; ...
   0.3016    200+ffr; ...
   0.4524    250+ffr; ...
   0.5655    300+ffr; ...
   1.1310    500+ffr; ...
   1.5080    600+ffr ]; 
 
% inverse damper characteristic (lookup table)  
damper_inv = [damper_table(:,2),damper_table(:,1)]; 

% topmount stiffness [N/m]
c = 400000;      
 
% define sweep sine excitation
f0 =  0.1;    % lowest frequency [Hz]
fe =   25;    % highest frequency [Hz]
n  =   50;    % number of frequency interval
amp=  0.01;   % amplitudes [m]

% sweep sine parameter
q = 1/n * log ( fe/f0 ) ;   p = q / ( f0 * ( 1 - (f0/fe)^(1/n) ) ) ;

% pre-allocate vars to speed up loop
freq=zeros(n+1,1); f0=zeros(n+1,1); fs=zeros(n+1,1); fc=zeros(n+1,1);

t0=0; s0=0; x0=s0;% initial conditions for first cycle
for ifreq=1:n+1

% Integrate from t0 to te results in  freq = 1/(te-t0)
  te=p/q*(1-exp(-ifreq*q));  
  freq(ifreq)=1/(te-t0); disp(['f=',num2str(freq(ifreq))]) 
  [t,xout] = ode23s(@damper_topmount_f,[t0,te],x0);  

% excitation  
  arg = 2*pi/q*log(p./(p-q*t)); se=sin(arg); ce=cos(arg);

% element force = force in topmount
  s = xout(:,1);   f = c*s; 

% curve fit to force (1. harmonic):  f = f0 + fs*sin(arg) + fc*cos(arg)
  A = [ones(size(arg))  se  ce  ] ;  coeff = A \ f ;         
  f0(ifreq) = coeff(1);  % offset
  fs(ifreq) = coeff(2);  % sine part
  fc(ifreq) = coeff(3);  % cosine part

% set initial conditions for next cycle  
  t0=te; x0=xout(length(t),:)';

end

% calculate dynamic stiffness and phase angle (tan(psi)=beta/alpha)
cdyn = sqrt(fs.^2+fc.^2)/amp;  psi = atan2(fc,fs)*180/pi;  

% plot frequence response
subplot(3,1,1), plot(freq,cdyn,'k','Linewidth',1), grid on 
subplot(3,1,2), plot(freq, psi,'k','Linewidth',1), grid on
subplot(3,1,3), plot(freq,  f0,'k','Linewidth',1), grid on