function xdot = qcm_nl_f(t,x)
% quarter car model with trailing arm suspension and nonlinear damper

global g s a b d e f h r0
global mC mK mW ThetaK ThetaW
global Ts0 cs  cx cz dx damper_table
global ustep tstep

% state variables
z = x(1);  beta = x(2);  phi = x(3);   zd = x(4);  betad = x(5);  phid = x(6);

% step input to actuator @ t = tstep
if t < tstep, u = 0; else u = ustep; end
                
% torque in revolute joint (preloaded linear torsional spring only)
Ts = - ( Ts0 + cs*beta );

% point to point damper element with non linear characteristic
rCDC=[-d*cos(beta);0;-b+d*sin(beta)]; rCEC=[-e;0;f]; % attachment points
rDEC = rCEC-rCDC; LDE=norm(rDEC); eDEC = rDEC/LDE; % action line
dvdbed=eDEC'*[d*sin(beta);0;d*cos(beta)]; v = dvdbed*betad; % velocity
FD=-interp1l(damper_table,v); qFDbe = dvdbed*FD; % element and general. force

% tire deflection (static tire radius)
rS = h + z - b + a*sin(beta) - u ;

% longitudinal tire force (adhesion assumed)
Fx = - cx*(a*(1-cos(beta))-rS*phi) - dx*(a*sin(beta)*betad-rS*phid);

% vertical tire force (contact assumed) 
Fz =  cz *( r0 - rS );  

% mass matrix
Massma=[       mC+mK+mW        (s*mK+a*mW)*cos(beta)    0   ; ...
        (s*mK+a*mW)*cos(beta)   ThetaK+s^2*mK+a^2*mW    0   ; ...
                  0                       0           ThetaW ];

% vector of generalized forces and torques
qgen=[ Fz-(mC+mK+mW)*g+(s*mK+a*mW)*sin(beta)*betad^2 ; ...     
       Ts-(s*mK+a*mW)*cos(beta)*g+a*(Fx*sin(beta)+Fz*cos(beta))+qFDbe; ...
         -rS*Fx ];
            
% state derivatives
xdot = [ zd;  betad;  phid;  Massma\qgen ];
                
end