function xdot = qcm_skyhook_f(t,x)

global  grav  M  m  cS kS  dCh dWh TdS  cT  
global  FS0  FT0  ftire  z_road
global  v_vel o_x0 o_y0 o_tp o_w o_h o_l

% get states
zc=x(1); zw=x(2);  zcd=x(3); zwd=x(4);  d=x(5);  

% obstacle @ sx = o_x0, sy = o_y0
sx = v_vel*t; sy=0; % actual position of vehicle
z_road = obstacle_f(sx-o_x0,sy-o_y0,o_tp,o_w,o_h,o_l); % road height

% tire force
ftire = max(0,FT0+cT*(z_road-zw)); 

% nonlinear spring
u  = zw - zc; % displacement (u>0: compression)
if u<0, 
  fs = FS0 + cS*u; % rebound
else
  fs = FS0 + cS*u*(1+kS*u^2); % compression
end

% damper velocity (v>0: compr.) and force via dynamic damping parameter
v = zwd - zcd;  fd = d*v;  

% compute skyhook damping parameter and its time derivative
if abs(v) > 0.001   
  d_sky = max(0,(dWh*zwd-dCh*zcd)/v); % nonlinear damping (d_sky>0)
else
  d_sky = d; % keep old value
end
ddot = (d_sky - d)/TdS;  % first order dynamics

% chassis and wheel acceleration
zcdd = (         (fs+fd) ) / M  -  grav ;
zwdd = ( ftire - (fs+fd) ) / m  -  grav ;

% state derivatives
xdot = [ zcd; zwd;  zcdd; zwdd;  ddot ];

end