function xp=vehicle_2d_simple_f(t,x)
% vehicle model including wheel/tire dynamics and simple ABS control

global  gravity mass theta a1 a2 h csf dsf csr dsr thetaw r  
global  fzn  dfx0 fxm sxm fxs sxs  vn  cx dx
global  t_tqb set_tqb  dn  dtqb_inc dtqb_dec
global  sx fx fz  tqba 

% get states
posx = x( 1); posz = x( 2); pitch = x( 3); % vehicle position and orientation
velx = x( 4); velz = x( 5); pdot  = x( 6); % vehicle velocities
owh = x( 7: 8); % angular velocities of front and rear wheels
xt  = x( 9:10); % longitudinal tire deflections
tqbctrl = x(11:12); % controlled braking torques

% wheel loads (linear suspension force characteristics) 
fz = [ mass*gravity*a2/(a1+a2)-csf*(posz-a1*pitch)-dsf*(velz-a1*pdot); ...
       mass*gravity*a1/(a1+a2)-csr*(posz+a2*pitch)-dsr*(velz+a2*pdot)];

% get maximum braking torque via linear interpolation
tqb_mx = interp1(t_tqb,set_tqb,t);

% tire and wheel  dynamics (front and rear wheel)
xtdot=[0;0]; fx=[0;0]; tqba=[0;0]; owhdot=[0;0];  % defaults
for i=1:2
  % modified transport velocities and long. slips without normalization
  vt = abs(r*owh(i))+vn;  vs=(velx-r*owh(i)); sx(i)=-vs/vt;
  % linear wheel load influence to tire parameter
  pt = [ dfx0, fxm, sxm, fxs, sxs] * fz(i)/fzn; 
  % generalized tire characteristics (sx only) 
  [f,fos] = tmy_fcombined( abs(sx(i)), pt(1),pt(2),pt(3),pt(4),pt(5) ) ;
  % time derivative of long. tire deflection and long. dynamic tire force
  xtdot(i) = -(cx*xt(i)*vt+fos*vs)/(dx*vt+fos);  fx(i)=cx*xt(i)+dx*xtdot(i);
  % limit braking torque to controlled or to maximum applied torque
  if abs(owh(i))>vn/r, tqb_l=tqbctrl(i); else  tqb_l=tqb_mx; end
  % applied braking torque (enhanced dry friction model)
  tqba(i)=-r*fx(i)+dn*owh(i);  tqba(i)=sign(tqba(i))*min(abs(tqba(i)),tqb_l);
  % angular momentum wheel (no driving torque)
  owhdot(i) = ( -tqba(i) - r*fx(i) ) / thetaw;    
end

% vehicle equations of motion (longitudinal and hub as well as pitch motion)
velxdot = ( fx(1) + fx(2) ) / mass;
velzdot = ( fz(1) + fz(2) ) / mass  -  gravity;   
pdotdot = (-a1*fz(1) + a2*fz(2) - (h+posz)*(fx(1)+fx(2)) )/theta;

% ABS control (simple 2-point brake torque control)
tqbctrldot=[0;0];  % no change in braking torques as default
for i=1:2
  if velx > vn % control braking torque only if vehicle is fast enough
    % increase up to the limit tqb_mx (normal braking)
    if tqbctrl(i) < tqb_mx, tqbctrldot(i) = dtqb_inc; end
    % decrease existing torque if slip is close to maximum (90% of sxm)
    if tqbctrl(i) > 0  &&  sx(i) < -0.9*sxm, tqbctrldot(i) = dtqb_dec; end
  end
end

% state derivatives
xp = [ velx;velz;pdot; velxdot;velzdot;pdotdot; owhdot; xtdot; tqbctrldot ]; 

end